@@ -6,13 +6,14 @@ Main purpose is to have a tested base.
 Submodules
 ----------
 
-.. toctree::
-   :maxdepth: 1
+.. autosummary::
+  :toctree: generated
 
    gftool.fourier
    gftool.lattice
    gftool.matrix
    gftool.pade
+   gftool.siam
 
 Glossary
 --------
@@ -48,291 +49,78 @@ import logging
 import warnings
 
 from typing import Callable
-from functools import partial
 from collections import namedtuple
 
 import numpy as np
-import scipy as sp
-import scipy.linalg
-import scipy.optimize
 
 from numpy import newaxis
-from scipy.special import expit, logit
+from scipy import optimize
 
-from . import lattice, matrix as gtmatrix
+from . import fourier, lattice, matrix as gtmatrix
+from ._util import _gu_sum
 from .basis import pole
 from ._version import get_versions
 
-__version__ = get_versions()['version']
-
-LOGGER = logging.getLogger(__name__)
-
 # Bethe lattice
 # pylint: disable=wrong-import-position
-from .lattice.bethe import (dos as bethe_dos,
-                            dos_moment as bethe_dos_moment,
-                            gf_d1_z as bethe_gf_d1_z,
-                            gf_d2_z as bethe_gf_d2_z,
+from .lattice.bethe import (dos as bethe_dos, dos_moment as bethe_dos_moment,
+                            gf_d1_z as bethe_gf_d1_z, gf_d2_z as bethe_gf_d2_z,
                             gf_z as bethe_gf_z,
                             hilbert_transform as bethe_hilbert_transform)
 
-# silence warnings of unused imports
-assert (bethe_dos and bethe_dos_moment and bethe_gf_d1_z and bethe_gf_d2_z
-        and bethe_gf_z and bethe_hilbert_transform)
-
-bethe_dos.m1 = partial(lattice.bethe.dos_moment, 1)
-bethe_dos.m2 = partial(lattice.bethe.dos_moment, 2)
-bethe_dos.m3 = partial(lattice.bethe.dos_moment, 3)
-bethe_dos.m4 = partial(lattice.bethe.dos_moment, 4)
-bethe_dos.m5 = partial(lattice.bethe.dos_moment, 5)
-
-
 # One-dimensional lattice
 from .lattice.onedim import (dos as onedim_dos,
                              dos_moment as onedim_dos_moment,
                              gf_z as onedim_gf_z,
                              hilbert_transform as onedim_hilbert_transform)
 
-# silence warnings of unused imports
-assert (onedim_dos and onedim_dos_moment and onedim_gf_z and onedim_hilbert_transform)
-
-onedim_dos.m1 = partial(lattice.onedim.dos_moment, 1)
-onedim_dos.m2 = partial(lattice.onedim.dos_moment, 2)
-onedim_dos.m3 = partial(lattice.onedim.dos_moment, 3)
-onedim_dos.m4 = partial(lattice.onedim.dos_moment, 4)
-onedim_dos.m5 = partial(lattice.onedim.dos_moment, 5)
-
-
 # Square lattice
 from .lattice.square import (dos as square_dos,
                              dos_moment as square_dos_moment,
                              gf_z as square_gf_z,
                              hilbert_transform as square_hilbert_transform)
 
+# Triangular lattice
+from .lattice.triangular import (dos as triangular_dos,
+                                 dos_moment as triangular_dos_moment,
+                                 gf_z as triangular_gf_z,
+                                 hilbert_transform as triangular_hilbert_transform)
+
+# Simple cubic lattice
+from .lattice.sc import (dos as sc_dos,
+                         dos_moment as sc_dos_moment,
+                         gf_z as sc_gf_z,
+                         hilbert_transform as sc_hilbert_transform)
+
+# Fermi statistics
+from .statistics import (fermi_fct, fermi_fct_d1, fermi_fct_inv,
+                         matsubara_frequencies, pade_frequencies)
+
+# Bose statistics
+from .statistics import (bose_fct, matsubara_frequencies_b)
+
+# Green's function give by finite number poles
+from .basis.pole import (gf_z as pole_gf_z,
+                         gf_d1_z as pole_gf_d1_z,
+                         gf_tau as pole_gf_tau,
+                         gf_ret_t as pole_gf_ret_t,
+                         moments as pole_gf_moments)
+
+__version__ = get_versions()['version']
+
+LOGGER = logging.getLogger(__name__)
+
 # silence warnings of unused imports
-assert (square_dos and square_dos_moment and square_gf_z and square_hilbert_transform)
-
-square_dos.m1 = partial(lattice.square.dos_moment, 1)
-square_dos.m2 = partial(lattice.square.dos_moment, 2)
-square_dos.m3 = partial(lattice.square.dos_moment, 3)
-square_dos.m4 = partial(lattice.square.dos_moment, 4)
-square_dos.m5 = partial(lattice.square.dos_moment, 5)
-
-
-def bose_fct(eps, beta):
-    r"""Return the Bose function `1/(exp(βϵ)-1)`.
-
-    Parameters
-    ----------
-    eps : complex or float or array_like
-        The energy at which the Bose function is evaluated.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    bose_fct : complex of float np.ndarray
-        The Bose function, same type as eps.
-
-    """
-    betaeps = np.asanyarray(beta*eps)
-    res = np.empty_like(betaeps)
-    small = betaeps < 700
-    res[small] = 1./np.expm1(betaeps[small])
-    # avoid overflows for big numbers using negative exponents
-    res[~small] = -np.exp(-betaeps[~small])/np.expm1(-betaeps[~small])
-    return res
-
-
-def fermi_fct(eps, beta):
-    r"""Return the Fermi function `1/(exp(βϵ)+1)`.
-
-    For complex inputs the function is not as accurate as for real inputs.
-
-    Parameters
-    ----------
-    eps : complex or float or ndarray
-        The energy at which the Fermi function is evaluated.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    fermi_fct : complex of float or ndarray
-        The Fermi function, same type as eps.
-
-    See Also
-    --------
-    fermi_fct_inv : The inverse of the Fermi function for real arguments
-
-    """
-    z = eps*beta
-    try:
-        return expit(-z)  # = 0.5 * (1. + tanh(-0.5 * beta * eps))
-    except TypeError:
-        pass  # complex arguments not handled by expit
-    z = np.asanyarray(z)
-    pos = z.real > 0
-    res = np.empty_like(z)
-    res[~pos] = 1./(np.exp(z[~pos]) + 1)
-    exp_m = np.exp(-z[pos])
-    res[pos] = exp_m/(1 + exp_m)
-    return res
-
-
-def fermi_fct_d1(eps, beta):
-    r"""Return the 1st derivative of the Fermi function.
-
-    .. math:: -β\exp(βϵ)/{(\exp(βϵ)+1)}^2
-
-    Parameters
-    ----------
-    eps : float or float ndarray
-        The energy at which the Fermi function is evaluated.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    fermi_fct_d1 : float or float ndarray
-        The Fermi function, same type as eps.
-
-    See Also
-    --------
-    fermi_fct
-
-    """
-    fermi = fermi_fct(eps, beta=beta)
-    return -beta*fermi*(1-fermi)
-
-
-def fermi_fct_inv(fermi, beta):
-    """Inverse of the Fermi function.
-
-    This is e.g. useful for integrals over the derivative of the Fermi function.
-
-    Parameters
-    ----------
-    fermi : float or float ndarray
-        The values of the Fermi function
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    fermi_fct_inv : float or float ndarray
-        The inverse of the Fermi function `fermi_fct(fermi_fct_inv, beta)=fermi`.
-
-    See Also
-    --------
-    fermi_fct
-
-    """
-    return -logit(fermi)/beta
-
-
-def matsubara_frequencies(n_points, beta):
-    r"""Return *fermionic* Matsubara frequencies :math:`iω_n` for the points `n_points`.
-
-    Parameters
-    ----------
-    n_points : int array_like
-        Points for which the Matsubara frequencies :math:`iω_n` are returned.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    matsubara_frequencies : complex ndarray
-        Array of the imaginary Matsubara frequencies
-
-    Examples
-    --------
-    >>> gt.matsubara_frequencies(range(1024), beta=1)
-    array([0.+3.14159265e+00j, 0.+9.42477796e+00j, 0.+1.57079633e+01j, ...,
-           0.+6.41827379e+03j, 0.+6.42455698e+03j, 0.+6.43084016e+03j])
-
-    """
-    n_points = np.asanyarray(n_points).astype(dtype=int, casting='safe')
-    return 1j * np.pi / beta * (2*n_points + 1)
-
-
-def matsubara_frequencies_b(n_points, beta):
-    r"""Return *bosonic* Matsubara frequencies :math:`iν_n` for the points `n_points`.
-
-    Parameters
-    ----------
-    n_points : int ndarray
-        Points for which the Matsubara frequencies :math:`iν_n` are returned.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    matsubara_frequencies : complex ndarray
-        Array of the imaginary Matsubara frequencies
-
-    Examples
-    --------
-    >>> gt.matsubara_frequencies_b(range(1024), beta=1)
-    array([0.+0.00000000e+00j, 0.+6.28318531e+00j, 0.+1.25663706e+01j, ...,
-           0.+6.41513220e+03j, 0.+6.42141538e+03j, 0.+6.42769857e+03j])
-
-    """
-    n_points = np.asanyarray(n_points).astype(dtype=int, casting='safe')
-    return 2j * np.pi / beta * n_points
-
-
-def pade_frequencies(num: int, beta):
-    """Return `num` *fermionic* Padé frequencies :math:`iz_p`.
-
-    The Padé frequencies are the poles of the approximation of the Fermi
-    function with `2*num` poles [ozaki2007]_.
-    This gives an non-equidistant mesh on the imaginary axis.
-
-    Parameters
-    ----------
-    num : int
-        Number of positive Padé frequencies.
-    beta : float
-        The inverse temperature :math:`beta = 1/k_B T`.
-
-    Returns
-    -------
-    izp : (num) complex np.ndarray
-        Positive Padé frequencies.
-    resids : (num) float np.ndarray
-        Residue of the Fermi function corresponding to `izp`. The residue is
-        given relative to true residue of the Fermi function corresponding to
-        the poles at Matsubara frequencies. This allows to use Padé frequencies
-        as drop-in replacement.
-        The actual residues would be `-resids/beta`.
-
-    References
-    ----------
-    .. [ozaki2007] Ozaki, Taisuke. Continued Fraction Representation of the
-       Fermi-Dirac Function for Large-Scale Electronic Structure Calculations.
-       Physical Review B 75, no. 3 (January 23, 2007): 035123.
-       https://doi.org/10.1103/PhysRevB.75.035123.
-
-    .. [hu2010] J. Hu, R.-X. Xu, and Y. Yan, “Communication: Padé spectrum
-       decomposition of Fermi function and Bose function,” J. Chem. Phys., vol.
-       133, no. 10, p.  101106, Sep. 2010, https://doi.org/10.1063/1.3484491
-
-    """
-    num = 2*num
-    a = -np.diagflat(range(1, 2*num, 2))
-    b = np.zeros_like(a, dtype=np.float_)
-    np.fill_diagonal(b[1:, :], 0.5)
-    np.fill_diagonal(b[:, 1:], 0.5)
-    eig, v = sp.linalg.eig(a, b=b, overwrite_a=True, overwrite_b=True)
-    sort = np.argsort(eig)
-    izp = 1j/beta * eig[sort]
-    resids = (0.25*v[0]*np.linalg.inv(v)[:, 0]*eig**2)[sort]
-    assert np.allclose(-izp[:num//2][::-1], izp[num//2:])
-    assert np.allclose(resids[:num//2][::-1], resids[num//2:])
-    assert np.all(~np.iscomplex(resids))
-    return izp[num//2:], resids.real[num//2:]
+assert all((fourier, lattice))
+assert all((bethe_dos, bethe_dos_moment, bethe_gf_d1_z, bethe_gf_d2_z,
+            bethe_gf_z, bethe_hilbert_transform))
+assert all((onedim_dos, onedim_dos_moment, onedim_gf_z, onedim_hilbert_transform))
+assert all((square_dos, square_dos_moment, square_gf_z, square_hilbert_transform))
+assert all((triangular_dos, triangular_dos_moment, triangular_gf_z, triangular_hilbert_transform))
+assert all((sc_dos, sc_dos_moment, sc_gf_z, sc_hilbert_transform))
+assert all((fermi_fct, fermi_fct_d1, fermi_fct_inv, matsubara_frequencies, pade_frequencies))
+assert all((bose_fct, matsubara_frequencies_b))
+assert all((pole_gf_z, pole_gf_d1_z, pole_gf_tau, pole_gf_ret_t, pole_gf_moments))
 
 
 def surface_gf_zeps(z, eps, hopping_nn):
@@ -423,7 +211,6 @@ def hubbard_dimer_gf_z(z, hopping, interaction, kind='+'):
     return gf_z
 
 
-# FIXME: write tests for moments
 def hubbard_I_self_z(z, U, occ):
     r"""Self-energy in Hubbard-I approximation (atomic solution).
 
@@ -465,11 +252,6 @@ def hubbard_I_self_z(z, U, occ):
     return hartree * z / (z - U + hartree)
 
 
-pole_gf_z = pole.gf_z
-pole_gf_d1_z = pole.gf_d1_z
-pole_gf_tau = pole.gf_tau
-
-
 def pole_gf_tau_b(tau, poles, weights, beta):
     """Bosonic imaginary time Green's function given by a finite number of `poles`.
 
@@ -525,10 +307,7 @@ def pole_gf_tau_b(tau, poles, weights, beta):
     if np.any(poles.real < 0):
         raise ValueError("Bosonic Green's function only well-defined for positive `poles`.")
     # eps((beta-tau)*pole)*g(pole, beta) = -exp(-tau*pole)*g(pole, -beta)
-    return np.sum(weights*bose_fct(poles, -beta)*np.exp(-tau*poles), axis=-1)
-
-
-pole_gf_moments = pole.moments
+    return _gu_sum(weights*bose_fct(poles, -beta)*np.exp(-tau*poles))
 
 
 def chemical_potential(occ_root: Callable[[float], float], mu0=0.0, step0=1.0, **kwds) -> float:
@@ -538,9 +317,9 @@ def chemical_potential(occ_root: Callable[[float], float], mu0=0.0, step0=1.0, *
     ----------
     occ_root : callable
         Function `occ_root(mu_i) -> occ_i - occ`, which returns the difference
-        in occupation for a chemical potential `mu_i` to the given occupation
-        `occ`. The sign is important, that `occ_i - occ` is returned.
-        Note that the sign is important!
+        in occupation to the desired occupation `occ` for a chemical potential
+        `mu_i`.
+        Note that the sign is important, `occ_i - occ` has to be returned!
     mu0 : float, optional
         The starting guess for the chemical potential. (default: 0)
     step0 : float, optional
@@ -554,6 +333,7 @@ def chemical_potential(occ_root: Callable[[float], float], mu0=0.0, step0=1.0, *
     Returns
     -------
     mu : float
+        The chemical potential given the correct charge `occ_root(mu)=0`.
 
     Raises
     ------
@@ -564,9 +344,9 @@ def chemical_potential(occ_root: Callable[[float], float], mu0=0.0, step0=1.0, *
 
     Notes
     -----
-    The search for a chemical potential is here a two-step procedure:
+    The search for a chemical potential is a two-step procedure:
     *First*, we search for a bracket `[mua, mub]` with
-    `occ_root(mua) < 0 < occ_root(mub)`. Here we use that the occupation is a
+    `occ_root(mua) < 0 < occ_root(mub)`. We use that the occupation is a
     monotonous increasing function of the chemical potential `mu`.
     *Second*, we perform a standard root-search in `[mua, mub]` which is done
     using `scipy.optimize.root_scalar`, Brent's method is currently used as
@@ -604,7 +384,7 @@ def chemical_potential(occ_root: Callable[[float], float], mu0=0.0, step0=1.0, *
             raise RuntimeError("No bracket `occ_root(mua) < 0 < occ_root(mub)` could be found.")
     bracket = list(sorted([mu1, mu0+step]))
     LOGGER.debug("Bracket found after %s iterations.", loops)
-    root_res = sp.optimize.root_scalar(occ_root, bracket=bracket, **kwds)
+    root_res = optimize.root_scalar(occ_root, bracket=bracket, **kwds)
     if not root_res.converged:
         runtime_err = RuntimeError(
             f"Root-search for chemical potential failed after {root_res.iterations}.\n"
@@ -626,6 +406,11 @@ def density(gf_iw, potential, beta, return_err=True, matrix=False, total=False):
     :math:`1/(iω_n + μ - ϵ - ℜΣ_{\text{static}})` will be calculated analytically.
     :math:`Σ_{\text{static}}` is the ω-independent mean-field part of the self-energy.
 
+    .. deprecated:: 0.8.0
+       Mostly superseded by more flexible `density_iw`, thus this function will
+       likely be discontinued. Currently `density` is a little more accurate
+       for `matrix=True`, compared to `density_iw` without using fitting.
+
     Parameters
     ----------
     gf_iw : complex ndarray
@@ -707,6 +492,8 @@ def density(gf_iw, potential, beta, return_err=True, matrix=False, total=False):
        1961): 942–49. https://doi.org/10.1103/PhysRev.121.942.
 
     """
+    warnings.warn("`density` is deprecated; use `density_iw` instead.",
+                  category=DeprecationWarning)
     iw = matsubara_frequencies(np.arange(gf_iw.shape[-1]), beta=beta)
     if total:
         assert gf_iw.ndim == 1
@@ -759,7 +546,7 @@ def density_iw(iws, gf_iw, beta, weights=1., moments=(1.,), n_fit=0):
         For Padé frequencies this needs to be provided.
     moments : (..., M) float array_like, optional
         Moments of the high-frequency expansion, where
-        `G(z) = moments / z**np.arange(N)` for large `z`.
+        `G(z) = np.sum(moments / z**np.arange(N))` for large `z`.
     n_fit : int, optional
         Number of additionally to `moments` fitted moments. If Padé frequencies
         are used, this is typically not necessary. (default: 0)
@@ -823,7 +610,7 @@ def density_iw(iws, gf_iw, beta, weights=1., moments=(1.,), n_fit=0):
     else:
         mom_gf = pole.PoleGf.from_moments(moments, width=None)
     delta_gf_iw = gf_iw.real - mom_gf.eval_z(iws).real
-    return 2./beta*np.sum(weights * delta_gf_iw.real, axis=-1) + mom_gf.occ(beta)[..., 0]
+    return 2./beta*_gu_sum(weights * delta_gf_iw.real) + mom_gf.occ(beta)[..., 0]
 
 
 def density_error(delta_gf_iw, iw_n, noisy=True):
