@@ -9,21 +9,22 @@ DOS.
 """
 import numpy as np
 
+from mpmath import mp
+
 from gftool.precision import PRECISE_TYPES as _PRECISE_TYPES
 
 
 def gf_z(z, half_bandwidth):
     r"""Local Green's function of Bethe lattice for infinite coordination number.
 
-    .. math::
-        G(z) = 2*(z - s\sqrt{z^2 - D^2})/D^2
+    .. math:: G(z) = 2(z - s\sqrt{z^2 - D^2})/D^2
 
     where :math:`D` is the half bandwidth and :math:`s=sgn[ℑ{ξ}]`. See
     [georges1996]_.
 
     Parameters
     ----------
-    z : complex ndarray or complex
+    z : complex array_like or complex
         Green's function is evaluated at complex frequency `z`
     half_bandwidth : float
         Half-bandwidth of the DOS of the Bethe lattice.
@@ -31,7 +32,7 @@ def gf_z(z, half_bandwidth):
 
     Returns
     -------
-    gf_z : complex ndarray or complex
+    gf_z : complex np.ndarray or complex
         Value of the Bethe Green's function
 
     References
@@ -48,11 +49,10 @@ def gf_z(z, half_bandwidth):
     >>> _ = plt.plot(ww, gf_ww.real, label=r"$\Re G$")
     >>> _ = plt.plot(ww, gf_ww.imag, '--', label=r"$\Im G$")
     >>> _ = plt.xlabel(r"$\omega/D$")
+    >>> _ = plt.xlabel(r"$G*D$")
     >>> _ = plt.axhline(0, color='black', linewidth=0.8)
-    >>> _ = plt.axvline(0, color='black', linewidth=0.8)
     >>> _ = plt.xlim(left=ww.min(), right=ww.max())
     >>> _ = plt.legend()
-    >>> plt.grid()
     >>> plt.show()
 
     """
@@ -70,7 +70,7 @@ def gf_d1_z(z, half_bandwidth):
 
     Parameters
     ----------
-    z : complex ndarray or complex
+    z : complex array_like or complex
         Green's function is evaluated at complex frequency `z`
     half_bandwidth : float
         half-bandwidth of the DOS of the Bethe lattice
@@ -78,7 +78,7 @@ def gf_d1_z(z, half_bandwidth):
 
     Returns
     -------
-    gf_d1_z : complex ndarray or complex
+    gf_d1_z : complex np.ndarray or complex
         Value of the derivative of the Green's function
 
     See Also
@@ -101,7 +101,7 @@ def gf_d2_z(z, half_bandwidth):
 
     Parameters
     ----------
-    z : complex ndarray or complex
+    z : complex array_like or complex
         Green's function is evaluated at complex frequency `z`
     half_bandwidth : float
         half-bandwidth of the DOS of the Bethe lattice
@@ -109,7 +109,7 @@ def gf_d2_z(z, half_bandwidth):
 
     Returns
     -------
-    gf_d2_z : complex ndarray or complex
+    gf_d2_z : complex np.ndarray or complex
         Value of the Green's function
 
     See Also
@@ -130,32 +130,30 @@ def gf_d2_z(z, half_bandwidth):
 def hilbert_transform(xi, half_bandwidth):
     r"""Hilbert transform of non-interacting DOS of the Bethe lattice.
 
-    The Hilbert transform is defined
+    The Hilbert transform is defined as:
 
-    .. math::
-        \tilde{D}(ξ) = ∫_{-∞}^{∞}dϵ \frac{DOS(ϵ)}{ξ − ϵ}
+    .. math:: \tilde{D}(ξ) = ∫_{-∞}^{∞}dϵ \frac{DOS(ϵ)}{ξ − ϵ}
 
     The lattice Hilbert transform is the same as the non-interacting Green's
     function.
 
     Parameters
     ----------
-    xi : complex ndarray or complex
+    xi : complex array_like or complex
         Point at which the Hilbert transform is evaluated
     half_bandwidth : float
         half-bandwidth of the DOS of the Bethe lattice
 
     Returns
     -------
-    hilbert_transform : complex ndarray or complex
+    hilbert_transform : complex np.ndarray or complex
         Hilbert transform of `xi`.
 
     Notes
     -----
-    Relation between nearest neighbor hopping `t` and half-bandwidth `D`
+    Relation between nearest neighbor hopping `t` and half-bandwidth `D`:
 
-    .. math::
-        2t = D
+    .. math:: 2t = D
 
     See Also
     --------
@@ -170,7 +168,7 @@ def dos(eps, half_bandwidth):
 
     Parameters
     ----------
-    eps : float ndarray or float
+    eps : float array_like or float
         DOS is evaluated at points `eps`.
     half_bandwidth : float
         Half-bandwidth of the DOS, DOS(| `eps` | > `half_bandwidth`) = 0.
@@ -178,9 +176,18 @@ def dos(eps, half_bandwidth):
 
     Returns
     -------
-    dos : float ndarray or float
+    dos : float np.ndarray or float
         The value of the DOS.
 
+    See Also
+    --------
+    gftool.lattice.bethe.dos_mp : multi-precision version suitable for integration
+
+    References
+    ----------
+    .. [economou2006] Economou, E. N. Green's Functions in Quantum Physics.
+       Springer, 2006.
+
     Examples
     --------
     >>> eps = np.linspace(-1.1, 1.1, num=500)
@@ -190,10 +197,9 @@ def dos(eps, half_bandwidth):
     >>> _ = plt.plot(eps, dos)
     >>> _ = plt.xlabel(r"$\epsilon/D$")
     >>> _ = plt.ylabel(r"DOS * $D$")
-    >>> _ = plt.axhline(0, color='black', linewidth=0.8)
     >>> _ = plt.axvline(0, color='black', linewidth=0.8)
+    >>> _ = plt.ylim(bottom=0)
     >>> _ = plt.xlim(left=eps.min(), right=eps.max())
-    >>> plt.grid()
     >>> plt.show()
 
     """
@@ -205,20 +211,22 @@ def dos(eps, half_bandwidth):
 
 
 # ∫dϵ ϵ^m DOS(ϵ) for half-bandwidth D=1
-# from:
-# https://www.wolframalpha.com/input/?i=integrate+sqrt(1+-+x%5E2)%2F(.5*pi)+*x%5E%7B2*n%7Ddx+from+-1+to+1+assuming+n+is+integer
+# from: integral of dos_mp with mp.workdps(100)
+# for m in range(0, 22, 2):
+#     with mp.workdps(100):
+#         print(mp.quad(lambda eps: 2 * eps**m * dos_mp(eps), [0, 1]))
+# rational numbers obtained by mp.identify
 dos_moment_coefficients = {
     2: 0.25,
     4: 0.125,
-    6: 0.078125,
-    8: 0.0546875,
-    # not sure if results form wolframalpha are accurate enough, calculated them with mpmath
-    # 10: 0.0410156,
-    # 12: 0.0322266,
-    # 14: 0.0261841,
-    # 16: 0.0218201,
-    # 18: 0.0185471,
-    # 20: 0.0160179,
+    6: 5/64,
+    8: 7/128,
+    10: 21/512,
+    12: 0.0322265625,
+    14: 0.02618408203125,
+    16: 0.021820068359375,
+    18: 0.01854705810546875,
+    20: 0.016017913818359375,
 }
 
 
@@ -255,3 +263,59 @@ def dos_moment(m, half_bandwidth):
         return dos_moment_coefficients[m] * half_bandwidth**m
     except KeyError as keyerr:
         raise NotImplementedError('Calculation of arbitrary moments not implemented.') from keyerr
+
+
+def dos_mp(eps, half_bandwidth=1):
+    r"""Multi-precision DOS of non-interacting Bethe lattice for infinite coordination number.
+
+    This function is particularly suited to calculate integrals of the form
+    :math:`∫dϵ DOS(ϵ)f(ϵ)`.
+
+    Parameters
+    ----------
+    eps : mpmath.mpf or mpf_like
+        DOS is evaluated at points `eps`.
+    half_bandwidth : mpmath.mpf or mpf_like
+        Half-bandwidth of the DOS, DOS(| `eps` | > `half_bandwidth`) = 0.
+        The `half_bandwidth` corresponds to the nearest neighbor hopping `t=D/2`
+
+    Returns
+    -------
+    dos_mp : mpmath.mpf
+        The value of the DOS.
+
+    See Also
+    --------
+    gftool.lattice.bethe.dos : vectorized version suitable for array evaluations
+
+    References
+    ----------
+    .. [economou2006] Economou, E. N. Green's Functions in Quantum Physics.
+       Springer, 2006.
+
+    Examples
+    --------
+    Calculate integrals:
+
+    >>> from mpmath import mp
+    >>> mp.quad(gt.lattice.bethe.dos_mp, [-1, 1])
+    mpf('1.0')
+
+    >>> eps = np.linspace(-1.1, 1.1, num=500)
+    >>> dos_mp = [gt.lattice.bethe.dos_mp(ee, half_bandwidth=1) for ee in eps]
+    >>> dos_mp = np.array(dos_mp, dtype=np.float64)
+
+    >>> import matplotlib.pyplot as plt
+    >>> _ = plt.plot(eps, dos_mp)
+    >>> _ = plt.xlabel(r"$\epsilon/D$")
+    >>> _ = plt.ylabel(r"DOS * $D$")
+    >>> _ = plt.axvline(0, color='black', linewidth=0.8)
+    >>> _ = plt.ylim(bottom=0)
+    >>> _ = plt.xlim(left=eps.min(), right=eps.max())
+    >>> plt.show()
+
+    """
+    eps, half_bandwidth = mp.mpf(eps), mp.mpf(half_bandwidth)
+    if mp.fabs(eps) > half_bandwidth:
+        return mp.mpf('0')
+    return 2 / (mp.pi * half_bandwidth) * mp.sqrt(-mp.powm1(eps / half_bandwidth, mp.mpf('2')))
